@@ -115,8 +115,10 @@ function paraselene_tag_position(tag,x,y){
 }
 
 function paraselene_tag_center(tag,x,y){
+	var sc_x=document.documentElement.scrollLeft || document.body.scrollLeft;
+	var sc_y=document.documentElement.scrollTop || document.body.scrollTop;
 	var size=new ParaseleneTagSize(tag);
-	paraselene_tag_position(tag,x-size.width/2,y-size.height/2);
+	paraselene_tag_position(tag,x-size.width/2+sc_x,y-size.height/2+sc_y);
 }
 
 function paraselene_tag_move(tag,x,y){
@@ -185,6 +187,25 @@ function paraselene_add_event(tag,kind,func){
 	}
 }
 
+function paraselene_remove_event(tag,kind,func){
+	if(tag.removeEventListener){
+		tag.removeEventListener(kind,func,false);
+	}
+	else{
+		tag.detachEvent("on"+kind,func);
+	}
+}
+
+function paraselene_safe_add_event(id,kind,func){
+	var tag=paraselene_get_id(id);
+	if(!tag)	return;
+	try{
+		paraselene_remove_event(tag,kind,func);
+	}
+	catch(e){}
+	paraselene_add_event(tag,kind,func);
+}
+
 function paraselene_kill_event(ev){
 	ev=paraselene_get_event(ev);
 	if(ev.preventDefault){
@@ -333,13 +354,13 @@ var paraselene_back_kill_f=false;
 var paraselene_comet_f=false;
 
 function paraselene_back_kill(){
-	if(!paraselene_back_kill_f)	return;
+	if(!paraselene_back_kill_f) return;
 	history.forward();
-	setTimeout(paraselene_back_kill,200);
+	setTimeout(paraselene_back_kill,10);
 }
 
 function paraselene_back_kill_start(frg){
-	if(paraselene_back_kill_f)	return;
+	if(paraselene_back_kill_f)return;
 	document.location.href=document.location.href+"#"+frg;
 	paraselene_back_kill_f=true;
 	paraselene_back_kill();
@@ -438,14 +459,14 @@ function paraselene_back_resize(){
 function paraselene_dialog_open(dialog){
 	var point=paraselene_get_id("paraseleneDragonsHead");
 	if(!point)	return;
+	var pageid=dialog.page_id;
 	if(dialog.active){
-		var act=paraselene_get_id(dialog.page_id);
+		var act=paraselene_get_id(pageid);
 		paraselene_z_order(act,false);
 		return;
 	}
 
 	var z=paraselene_popup.length+dialog.base_z+1;
-	var pageid=dialog.page_id;
 	if(dialog.modal==true){
 		var back=paraselene_get_id("paraseleneEclipse");
 		if(paraselene_modal_count<1){
@@ -503,6 +524,7 @@ function paraselene_action(obj){
 	for(var i=0;i<obj.data.length;i++){
 		var val=obj.data[i];
 		if(val.location){
+			paraselene_back_kill_stop();
 			window.location.href=val.location;
 			continue;
 		}
@@ -597,7 +619,7 @@ function paraselene_get_a(o,url,key){
 
 function paraselene_agency_a(o,key){
 	var href=o.getAttribute("href").split("?");
-	href[0]=href[0]+"?paraselene_a_dummy="+paraselene_make_dummy();
+	href[0]=href[0]+"?paraselene_dummy="+paraselene_make_dummy();
 	if(href[1]){
 		href[0]=href[0]+"&"+href[1];
 	}
@@ -619,7 +641,7 @@ function paraselene_agency_form(o,key){
 	if(!url)	return true;
 	o.target="paraseleneDragonsTail";
 	o.paraselene_form_ajax.value=key;
-	o.paraselene_form_dummy.value=paraselene_make_dummy();
+	o.paraselene_dummy.value=paraselene_make_dummy();
 	paraselene_called_form=o;
 	return true;
 }
