@@ -8,12 +8,20 @@ import paraselene.tag.*;
 import paraselene.tag.form.*;
 import paraselene.tag.list.*;
 import paraselene.tag.table.*;
+import paraselene.dyna.*;
 
 /**
  * サイトページ基底クラス。
  * 全てのページの共通メソッドを持たせたい場合にはこのクラスに記述できます。
  */
 public abstract class SuperPage extends Page {
+	/**
+	 * name 属性の検証。
+	 * @param page 検証するページ
+	 * @param exclude 検証から除外する name。
+	 * @return エラーとなった name。
+	 */
+	public abstract NameDefine[] inspectName( Page page, String ... exclude );
 	/**
 	 * 入力値の検証メイン処理。
 	 * @param req リクエスト内容。
