@@ -198,7 +198,7 @@ public class Text extends PlainText implements AttributeValuable, CSSValuable {
 			if ( mode != StringMode.ATTRIBUTE ) {
 				WebChar	wc = WebChar.getWebChar( line[i] );
 				if ( wc != null ) {
-					buf = buf.append( wc.getRef() );
+					buf = buf.append( wc.getReference() );
 					continue;
 				}
 				if ( mode != StringMode.TEXTAREA ) {
