@@ -18,9 +18,9 @@ public class Version {
 	private Version(){}
 	/**
 	 * Paraseleneのバージョン番号を表す文字列。
-	 * 2.1.1.16
+	 * 2.1.1.17
 	 */
-	public static final String VERSION = "2.1.1.16";
+	public static final String VERSION = "2.1.1.17";
 	private static final String	SITE = " (http://paraselene.sourceforge.jp/)";
 	private static final String	TITLE = "Paraselene2 " + VERSION + SITE;
 	private static final String[] CLASS_PATH = new String[] {
@@ -113,6 +113,14 @@ public class Version {
 	 * @return リソース。取得に失敗すると null を返します。
 	 */
 	public static InputStream getResource( String name ) {
+		String[]	test = name.split( "\\\\" );
+		StringBuilder	buf = new StringBuilder( test[0] );
+		for ( int i = 1; i < test.length; i++ ) {
+			buf = buf.append( "/" );
+			buf = buf.append( test[i] );
+		}
+		name = buf.toString();
+
 		ClassLoader	ld = Version.class.getClassLoader();
 		if ( ld != null ) {
 			InputStream	is = ld.getResourceAsStream( name );
