@@ -37,13 +37,6 @@ class URIEcho implements URIResolver {
  * 上記理由により、PageIDを持ちませんし、履歴管理もされません。<br>
  * またHTMLパース処理を伴うため、通常のモックアップHTML由来のPage生成に
  * 比べ、大幅に処理コストがかかります。その点に留意して使用して下さい。<br>
- * スケルトン生成ツールと同様にDTDのキャッシュ機能が働きます。<br>
- * XHTMLは厳密に解釈するため例外が発生する可能性があります。<br>
- * 一方HTMLはルーズな解釈です。このため、意図したデザインと異なる解釈を行う
- * 可能性があります。<br>
- * 用意したHTMLファイルが例外を出さずにパース可能かを検証するために、
- * {@link paraselene.dyna.DynamicPage#main}メソッドを持っています。
- * リリース前検証に利用して下さい。<br>
  * name属性値が付いたタグは{@link paraselene.Page#getTag}等でアクセス可能です。
  */
 public class DynamicPage extends Page {
@@ -422,43 +415,62 @@ public class DynamicPage extends Page {
 	public int getUploadMaxBytes() { return -1; }
 	public boolean isCheckRepeatSameRequest() { return false; }
 
-	/**
-	 * 解析のテスト。
-	 * 本番環境にHTMLファイルを配置してシステムに解析させる前に、解析が可能か？
-	 * どのようなURIが含まれるか？どのようなname属性値を持つか？
-	 * を検査する事ができます。<br>
-	 * 起動パラメータは、<br>
-	 * 検査するHTMLファイル名<br>
-	 * ファイルの文字コード<br>
-	 * の２つを渡して下さい。
-	 * @param argv 検査するHTMLファイル名 ファイルの文字コード
-	 */
-	public static void main( String[] argv ) throws Exception {
-		System.out.println( Version.getTitle() );
-		System.out.println( Version.getCopyRight() );
-		if ( argv.length != 2 ) {
-			System.out.println( "HTMLファイル名 ファイルの文字コード" );
-			System.out.println( "を指定して下さい。" );
-			System.exit( 0 );
-		}
-		DynamicPage	page = new DynamicPage( new URIEcho(), null );
-		page.create( new File( argv[0] ), argv[1] );
-		String[]	name = page.getDefineNameIndex();
-		for ( int i = 0; i < name.length; i++ ) {
-			System.out.print( "name属性: " );
-			System.out.println( name[i] );
-			Tag[]	tag = page.getAllTag( name[i] );
-			for ( int j = 0; j < tag.length; j++ ) {
-				System.out.print( " -> " );
-				System.out.println( tag[j].getClass().getName() );
-			}
-		}
-		System.out.println( "正常に解析可能です" );
-		System.exit( 0 );
-	}
-
 	public AjaxSupport getAjaxSupport() { return AjaxSupport.NO; }
 
 	public void firstOutput( RequestParameter req ) throws PageException{}
+
+	/**
+	 * name属性値検証。<br>
+	 * define で指定された name 属性値が page に登場しているか検査します。<br>
+	 * define で指定したうち、検査対象から外したいものがあれば、exclude に
+	 * 除外する name 属性値を指定する事ができます。<br>
+	 * 登場しない name は戻り値で返されます。
+	 * 全ての指定 name が存在すれば null を返します。<br>
+	 * define にクラス指定があれば、クラスが一致しなければエラーとなります。
+	 * @param page 検証対象ページ。
+	 * @param define 検証する name 属性値。
+	 * @param exclude 検証から除外する name 属性値。
+	 * @return エラーがあった name 属性値。全て登場していれば null。
+	 */
+	public static NameDefine[] inspectName( Page page, NameDefine[] define, String ... exclude ) {
+		if ( define == null )	return null;
+		ArrayList<NameDefine>	err = new ArrayList<NameDefine>();
+		for ( int i = 0; i < define.length; i++ ) {
+			define[i].setError( "OK." );
+			boolean	flag = false;
+			String	name = define[i].getName();
+			for ( int j = 0; j < exclude.length; j++ ) {
+				if ( name.equals( exclude[j] ) ) {
+					define[i].setError( "It is excluded." );
+					flag = true;
+					break;
+				}
+			}
+			if ( flag )	continue;
+
+			Tag[]	tag = page.getAllTag( name );
+			if ( tag == null || tag.length == 0 ) {
+				define[i].setError( "Name is undefined." );
+				err.add( define[i] );
+				continue;
+			}
+			Class<?>	cls = define[i].getDefineClass();
+			if ( cls == null )	continue;
+			flag = false;
+			for ( int j = 0; j < tag.length; j++ ) {
+				if ( cls.equals( tag[j].getClass() ) ) {
+					flag = true;
+					break;
+				}
+			}
+			if ( !flag ) {
+				define[i].setError( "Class is different." );
+				err.add( define[i] );
+			}
+		}
+		if ( err.size() == 0 )	return null;
+		return err.toArray( new NameDefine[0] );
+	}
+
 }
 
