@@ -99,6 +99,9 @@ public class Forward implements Serializable {
 	 * @param af 動作指定。
 	 */
 	public Forward( AjaxForward ... af ) {
+		if ( af.length == 0 ) {
+			af = new AjaxForward[]{ new NOP() };
+		}
 		ajax = trim( af, SandBox.getCurrentRequestParameter().getHistory(), true );
 		not_ajax = getNotAjax();
 	}
@@ -220,7 +223,7 @@ public class Forward implements Serializable {
 		done_f = true;
 		if ( error != 0 ) {
 			if ( error == 403 ) {
-//Thread.dumpStack();
+				Option.traceWithStack( "403 Forward" );
 			}
 			res.sendError( error );
 			return;
