@@ -46,13 +46,20 @@ class HistorySet implements Serializable {
 
 	synchronized void drop() {
 		set_no.clear();
-		for ( int key: hist.keySet() ) {
-			remove( key );
+		boolean flag = true;
+		while( flag ){
+			flag = false;
+			for ( int key: hist.keySet() ) {
+				remove( key );
+				flag = true;
+				break;
+			}
 		}
 		Option.trace( "history all drop" );
 	}
 
 	synchronized private int newNo( int no ) {
+		if ( no < 0 )	no *= -1;
 		while ( true ) {
 			if ( no == RAND || hist.get( no ) != null ) {
 				no++;
