@@ -37,12 +37,102 @@ class RedirectData implements Serializable {
 }
 
 class SessionData implements Serializable, HttpSessionBindingListener {
-	private static final long serialVersionUID = 2L;
+	private static final long serialVersionUID = 25L;
 	TransactionSequencer seq = new TransactionSequencer();
 	HistorySet hist = null;
 	private HashMap<PageID, RedirectData>	pending = new HashMap<PageID, RedirectData>();
 	private volatile int client;
 
+	private class Waiter implements Serializable {
+		private static final long serialVersionUID = 25L;
+		volatile boolean flag = true;
+	}
+	private static final String	DUMMY = "paraselene_dmy";
+	private ArrayList<String>	cache_dummy_key = new ArrayList<String>();
+	private ArrayList<CacheAjax>	cache_ajax = new ArrayList<CacheAjax>();
+	private HashMap<String, Waiter>	working_ajax = new HashMap<String, Waiter>();
+
+	private String getCacheKey( RequestParameter param ) {
+		RequestItem	item = param.getItem( DUMMY );
+		if ( item == null )	return null;
+		String	key = item.getValue( 0 );
+		if ( key == null )	return null;
+		key = key.trim();
+		if ( key.isEmpty() )	return null;
+		return key;
+	}
+
+	void cacheEntry( RequestParameter param, CacheAjax ca ) {
+		String	key = getCacheKey( param );
+		if ( key == null )	return;
+		synchronized( cache_dummy_key ) {
+			cache_dummy_key.add( key );
+			cache_ajax.add( ca );
+			if ( cache_dummy_key.size() > 5 ) {
+				cache_dummy_key.remove( 0 );
+				cache_ajax.remove( 0 );
+			}
+		}
+		cacheUnlock( param );
+	}
+
+	void cacheUnlock( RequestParameter param ) {
+		if ( param == null )	return;
+		String	key = getCacheKey( param );
+		if ( key == null )	return;
+		Waiter	o = null;
+		synchronized( working_ajax ) {
+			o = working_ajax.remove( key );
+		}
+		if ( o == null )	return;
+		synchronized( o ) {
+			o.flag = false;
+			Option.trace( "%s release", key );
+			o.notifyAll();
+		}
+	}
+
+	Forward cacheResponse( RequestParameter param, HttpServletResponse res ) throws IOException {
+		String	key = getCacheKey( param );
+		if ( key == null )	return null;
+		Waiter	o1 = null, o2 = new Waiter();
+		synchronized( working_ajax ) {
+			o1 = working_ajax.get( key );
+			if ( o1 == null ) {
+				Option.trace( "%s ajax lock", key );
+				working_ajax.put( key, o2 );
+			}
+		}
+		if ( o1 != null ) {
+			synchronized( o1 ) {
+				if ( o1.flag ) {
+					Option.trace( "%s waiting", key );
+					while( true ){
+						try {
+							o1.wait();
+							break;
+						}
+						catch( InterruptedException e ){}
+					}
+				}
+			}
+			return cacheResponse( param, res );
+		}
+		synchronized( cache_dummy_key ) {
+			for ( int i = cache_dummy_key.size() - 1; i >= 0; i-- ) {
+				String	ck = cache_dummy_key.get( i );
+				if ( ck.equals( key ) ) {
+					cacheUnlock( param );
+					CacheAjax	ca = cache_ajax.get( i );
+					ca.writeTo( res );
+					Option.trace( "used cache response" );
+					return new NullForward();
+				}
+			}
+		}
+		return null;
+	}
+
 	private String cut( String org, int len ) {
 		if ( org == null )	return null;
 		int	org_len = org.length();
