@@ -360,14 +360,14 @@ public abstract class Supervisor extends HttpServlet {
 			if ( path_data == null ) {
 				fw = new Forward( 403 );
 				fw.Action( this, res );
-				clear_session( session );
+				//clear_session( session );
 				return;
 			}
 			if ( path_data.redirect != null ) {
 				if ( !holder.data.write( this, path_data.redirect, holder ) ) {
 					fw = new Forward( 403 );
 					fw.Action( this, res );
-					clear_session( session );
+					//clear_session( session );
 				}
 				return;
 			}
@@ -634,7 +634,7 @@ public abstract class Supervisor extends HttpServlet {
 			data.redirect( out_ret, fw, param ).Action( this, res );
 		}
 		else {
-			String	mime = next_page.getContentType();
+			String	mime = out_ret[1].getContentType();
 			if ( mime == null ) {
 				StringBuilder	buf = null;
 				if ( next_page.isXML() ) {
@@ -691,58 +691,31 @@ public abstract class Supervisor extends HttpServlet {
 					res.addCookie( fw.out_cookie.get( i ) );
 				}
 			}
-			if ( gzip( param, holder.req, res, out_ret[1] ) ) {
+			if ( gzip( param, holder.req, res, out_ret[1], data, mime ) ) {
 				data.hist.get( next_page.getHistoryKey() ).add( next_page );
 				return;
 			}
 			if ( out_ret[1] instanceof Downloadable ) {
-				//res.addHeader( "Transfer-Encoding", "chunked" );
-				Downloadable	dl = (Downloadable)out_ret[1];
-				OutputStream	out = new BufferedOutputStream( res.getOutputStream() );
-				BufferedInputStream		in = new BufferedInputStream( dl.getInputStream() );
-				try{
-					//byte[]	crlf = new byte[] { 0x0d, 0x0a };
-					byte[]	buf = new byte[4096];
-					while( true ) {
-						int	size = in.read( buf );
-						if ( size <= 0 )	break;
-						//out.write( Integer.toString( size, 16 ).getBytes() );
-						//out.write( crlf );
-						out.write( buf, 0, size );
-						//out.write( crlf );
-						if ( size < buf.length )	break;
-						holder.req.getSession( false );
-					}
-					//out.write( Integer.toString( 0, 16 ).getBytes() );
-					//out.write( crlf );
-					//out.write( crlf );
-					in.close();
-					out.flush();
-				}
-				catch( Exception e ) {
-					if ( in != null ) {
-						in.close();
-					}
-					throw e;
-				}
+				CacheBin	cb = new CacheBin( mime, false );
+				cb.write( (Downloadable)out_ret[1], param, data, res );
 			}
 			else {
-				PrintWriter	w = res.getWriter();
+				CacheWriter	w = new CacheWriter( out_ret[1], mime, param, data );
 				History	h = data.hist.get( out_ret[1].getHistoryKey() );
 				try {
-					h.lock();
+					if ( h != null )	h.lock();
 					out_ret[1].write( w );
 				}
 				finally{
-					h.unlock();
+					if ( h != null )	h.unlock();
 				}
-				w.flush();
+				w.writeTo( res );
 			}
 		}
 		data.hist.get( next_page.getHistoryKey() ).add( next_page );
 	}
 
-	private boolean gzip( RequestParameter param, HttpServletRequest req, HttpServletResponse res, Page page )  throws Exception {
+	private boolean gzip( RequestParameter param, HttpServletRequest req, HttpServletResponse res, Page page, SessionData data, String mime )  throws Exception {
 		if ( !page.isGZIP() )	return false;
 		String[]	pcol = req.getProtocol().split( "\\/" );
 		if ( pcol[pcol.length - 1].equals( "1.0" ) )	return false;
@@ -765,43 +738,30 @@ public abstract class Supervisor extends HttpServlet {
 		if ( !gzip_f )	return false;
 
 		res.addHeader( "Content-Encoding", "gzip" );
-		GZIPOutputStream	gz_out = new GZIPOutputStream( res.getOutputStream() );
 		if ( page instanceof Downloadable ) {
-			Downloadable	dl = (Downloadable)page;
-			BufferedInputStream		in = new BufferedInputStream( dl.getInputStream() );
-			try {
-				byte[]	buf = new byte[4096];
-				while( true ) {
-					int	size = in.read( buf );
-					if ( size <= 0 )	break;
-					gz_out.write( buf, 0, size );
-					if ( size < buf.length )	break;
-					req.getSession( false );
-				}
-				in.close();
-			}
-			catch( Exception e ) {
-				if ( in != null )	in.close();
-				throw e;
-			}
+			CacheBin	cb = new CacheBin( mime, true );
+			cb.write( (Downloadable)page, param, data, res );
 		}
 		else {
-			PrintWriter	w = new PrintWriter(
+			CacheWriter	w = new CacheWriter( page, mime, param, data );
+			History	h = data.hist.get( page.getHistoryKey() );
+			try {
+				if ( h != null )	h.lock();
+				page.write( w );
+			}
+			finally {
+				if ( h != null )	h.unlock();
+			}
+			GZIPOutputStream	gz_out = new GZIPOutputStream( res.getOutputStream() );
+			w.writeTo( res, new PrintWriter(
 				new OutputStreamWriter(
 					gz_out,
 					page.getCharset()
-				)
+				) )
 			);
-			page.write( w );
-			w.flush();
+			gz_out.finish();
 		}
-		gz_out.finish();
 
-/*
-		res.addHeader( "Content-Length", Integer.toString( b_out.size() ) );
-		BufferedOutputStream	out = new BufferedOutputStream( res.getOutputStream() );
-		b_out.writeTo( out );
-*/
 		return true;
 	}
 
@@ -824,7 +784,7 @@ public abstract class Supervisor extends HttpServlet {
 	 */
 	public abstract int getRequestMaxCount();
 
-	private Forward preset( RequestParameter.Method method, HttpServletRequest request, HttpServletResponse res ) throws UnsupportedEncodingException {
+	private Forward preset( RequestParameter.Method method, HttpServletRequest request, HttpServletResponse res ) throws UnsupportedEncodingException, IOException {
 		if ( !SandBox.isSafeFreeMemory() ) {
 			return new Forward( 503 );
 		}
@@ -889,6 +849,9 @@ public abstract class Supervisor extends HttpServlet {
 		}
 		Option.appendTrace( log.toArray( new String[0] ) );
 
+		Forward	ret = data.cacheResponse( req, res );
+		if ( ret != null )	return ret;
+
 		holder.search_engine = req.judgeSearchEngine() != RequestParameter.SearchEngine.NO_SEARCHENGINE;
 		holder.request = req;
 
@@ -922,14 +885,17 @@ public abstract class Supervisor extends HttpServlet {
 				return;
 			}
 			doAction( req, res );
+		}
+		catch( Exception e ) {
+			onErrorDebug( e, req, res );
+		}
+		finally {
+			cacheUnlock();
 			synchronized( data_map_th ) {
 				data_map_th.remove( Thread.currentThread() );
 				data_map_sv.remove( this );
 			}
 		}
-		catch( Exception e ) {
-			onErrorDebug( e, req, res );
-		}
 	}
 
 	protected final void doPost( HttpServletRequest req, HttpServletResponse res )  {
@@ -940,14 +906,33 @@ public abstract class Supervisor extends HttpServlet {
 				return;
 			}
 			doAction( req, res );
+		}
+		catch( Exception e ) {
+			onErrorDebug( e, req, res );
+		}
+		finally {
+			cacheUnlock();
 			synchronized( data_map_th ) {
 				data_map_th.remove( Thread.currentThread() );
 				data_map_sv.remove( this );
 			}
 		}
-		catch( Exception e ) {
-			onErrorDebug( e, req, res );
+	}
+
+	private void cacheUnlock() {
+		SessionData	data = null;
+		DataHolder	holder = null;
+		RequestParameter	req = null;
+		synchronized( data_map_th ) {
+			holder = data_map_th.get( Thread.currentThread() );
+			if ( holder == null )	holder = data_map_sv.get( this );
+			if ( holder == null )	return;
 		}
+		data = holder.data;
+		if ( data == null )	return;
+		req = holder.request;
+		if ( req == null )	return;
+		data.cacheUnlock( req );
 	}
 }
 
