@@ -81,6 +81,7 @@ public class Tag extends HTMLPart {
 	}
 
 	private void set_modify() {
+		//if ( "body".equals( name ) )	return;
 		if ( getAttribute( "id" ) == null ) {
 			Tag	out = getOuterTag();
 			if ( out != null ) {
@@ -221,9 +222,10 @@ public class Tag extends HTMLPart {
 	 * このタグに重複しないID属性を設定します。<br>
 	 * ただし、既にID属性も持っている場合は何もしません。
 	 */
-	public void makeID() {
-		if ( getAttribute( "id" ) != null )	return;
-		if ( "head".equals( getName() ) )	return;
+	public String makeID() {
+		Attribute	id = getAttribute( "id" );
+		if ( id != null )	return id.getString();
+		if ( "head".equals( getName() ) )	return null;
 		long	now = 0;
 		synchronized( PARASELENE_ID ) {
 			now = tag_id;
@@ -233,7 +235,9 @@ public class Tag extends HTMLPart {
 		time &= 0x7fffffffffffffffL;
 		StringBuilder	buf = new StringBuilder( PARASELENE_ID );
 		buf = buf.append( Long.toString( time, Character.MAX_RADIX ) );
-		setAttribute( new Attribute( "id", buf.toString() ) );
+		String	nid = buf.toString();
+		setAttribute( new Attribute( "id", nid ) );
+		return nid;
 	}
 
 	/**
@@ -254,8 +258,8 @@ public class Tag extends HTMLPart {
 	public Tag( String n, boolean simple_f ) {
 		this();
 		name = n.toLowerCase( Locale.ENGLISH );
-		nest_f = !simple_f;
 		if ( "body".equals( name ) )	makeID();
+		nest_f = !simple_f;
 	}
 
 	/**
@@ -1143,6 +1147,64 @@ public class Tag extends HTMLPart {
 		return null;
 	}
 
+	/**
+	 * イベントハンドラの登録。
+	 * このタグが存在するページの、{@link Page#addOnLoadScript}を
+	 * 呼び出します。そのため、ページに追加されたタグでなければなりません。<br>
+	 * また、上と同じ理由で、ページ出力されると内容がクリアされます。
+	 * @param triger &quot;click&quot;や&quot;submit&quot;等。
+	 * &quot;on&quot;を付けないで下さい。
+	 * @param function JavaScript関数名。()や;を付けないで下さい。
+	 */
+	public void setEventHandler( String triger, String function ) {
+		Page	page = getAssignedPage();
+		String	id = makeID();
+		StringBuilder	buf = new StringBuilder();
+		buf = buf.append( "paraselene_safe_add_event('" );
+		buf = buf.append( id );
+		buf = buf.append( "','" );
+		buf = buf.append( triger );
+		buf = buf.append( "'," );
+		buf = buf.append( function );
+		buf = buf.append( ");" );
+		page.addOnLoadScript( buf.toString() );
+	}
+
+	/**
+	 * イベントハンドラの登録。
+	 * {@link #setEventHandler}と異なり、無名関数を登録します。
+	 * @param triger &quot;click&quot;や&quot;submit&quot;等。
+	 * &quot;on&quot;を付けないで下さい。
+	 * @param function JavaScript無名関数。例えば、<pre>
+	 * &quot;function(event){&quot;,
+	 * &quot;  alert( 'CLICK' );&quot;,
+	 * &quot;}&quot;
+	 * </pre>のような文字列群です。各文字列には改行が追加されます。
+	 */
+	public void setAnonymousEventHandler( String triger, String ... function ) {
+		String	id = makeID();
+		StringBuilder	script = new StringBuilder();
+		for ( int i = 0; i < function.length; i++ ) {
+			if ( i > 0 )	script = script.append( "\r\n" );
+			script = script.append( function[i] );
+		}
+		String	ano = script.toString();
+		StringBuilder	buf = new StringBuilder( "paraselene_handler_" );
+		buf = buf.append( id );
+		int	hc = ano.hashCode();
+		if ( hc < 0 )	hc *= -1;
+		buf = buf.append( Integer.toString( hc, Character.MAX_RADIX ) );
+		ano = buf.toString();
+		buf = new StringBuilder( "var " );
+		buf = buf.append( ano );
+		buf = buf.append( "=" );
+		buf = buf.append( script );
+		buf = buf.append( ";" );
+		getAssignedPage().addOnLoadScript( buf.toString() );
+		setEventHandler( triger, ano );
+	}
+
+/************
 	protected void setButtonAjaxHandler( String triger, String function, Object ... param ) {
 		Page	page = getAssignedPage();
 		if ( !page.isAjax() )	return;
@@ -1190,22 +1252,21 @@ public class Tag extends HTMLPart {
 		Tag	tag = makeScript( s );
 		tag.write( w, mode );
 	}
+*******/
+
 
 	/**
-	 * JavaScript を安全に記述するための script タグを生成します。
+	 * JavaScript を安全に記述するための script タグを生成します。<br>
+	 * このタグの内容を変更しても、Feedback として通知されないので注意して下さい。
 	 * @param script JavaScript。各引数には改行が挟まれて出力されます。
 	 * @return script タグ。
 	 */
 	public static Tag makeScript( String ... script ) {
-		Tag	tag = new Tag( "script", false );
+		Tag	tag = new Tag( "script", false ) {
+			public void setModify() {return;}
+		};
 		tag.setAttribute( new Attribute( "type", "text/javascript" ) );
-		StringBuilder	buf = new StringBuilder( "<!--//--><![CDATA[//><!--\r\n" );
-		for ( int i = 0; i < script.length; i++ ) {
-			buf = buf.append( script[i] );
-			buf = buf.append( "\r\n" );
-		}
-		buf = buf.append( "//--><!]]>\r\n" );
-		tag.addHTMLPart( new PlainText( buf.toString() ) );
+		tag.addHTMLPart( new paraselene.ajax.data.JavaScript( script ) );
 		return tag;
 	}
 }
