@@ -45,7 +45,19 @@ public enum WebChar {
 		return new String( new char[]{ ch } );
 	}
 
-	String getRef() {
+	/**
+	 * 文字化。１文字のcharとして扱う事ができます。
+	 * @return 文字。
+	 */
+	public char toChar() {
+		return ch;
+	}
+
+	/**
+	 * 文字参照の取得。&amp;nbsp; のような文字列を返します。
+	 * @return 文字参照。
+	 */
+	public String getReference() {
 		return ref;
 	}
 }
